
/*
  Copyright (C) 1997, IGS Technology, Inc.

  Permission is hereby granted to use, copy and modify this software
  only for the purpose of developing IGS Technology related products.

  This software may not be distributed to any party which is not covered
  by IGS Technology Non-Disclosure Agreement (NDA).

  This software is provided "as is" without warranty of any kind,
  either expressed or implied, including but not limited to the implied
  warranties of merchantability and/or fitness for a particular purpose.

  Author: Bo


		 Video Overlay FIFO Tables
		 -------------------------
  These tables are used to program some Video FIFO related registers.
  As the fact, video FIFO is relolution, color depth, refresh rate, memory size,
  and memory clock dependant. Theoretically, one FIFO table can cover all the
  combinations of the above factors, but the results will not be as good as
  desired. As the result, IGS comes out a few FIFO tables to cover all conditions.
  It might be possible to have more FIFO tables under some circumstances.

*/

#ifndef _FIFOTAB_H
#define _FIFOTAB_H

unsigned short wFIFOTable0[] = {
		   OFF, 	  /* Vertical Filter on or off */
		   0x1074,	  /* 3CE_Reg_74     */
		   0x1075,	  /* 3CE_Reg_75     */
		   0xF8D9,	  /* 3CE_Reg_D9     */
		   0x7CDA,	  /* 3CE_Reg_DA     */
		   0x02DD	  /* 3CE_Reg_DD     */
		 };

unsigned short wFIFOTable1[] = {
		   ON,
		   0x1074,
		   0x1075,
		   0x3CD9,
		   0x08DA,
		   0x02DD
		 };

unsigned short wFIFOTable2[] = {
		   OFF,
		   0x1C74,
		   0x1C75,
		   0xF8D9,
		   0x7CDA,
		   0x02DD
		 };

unsigned short wFIFOTable3[] = {
		   OFF,
		   0x1074,
		   0x1075,
		   0x6FD9,
		   0xF8DA,
		   0x06DD
		 };

unsigned short wFIFOTable4[] = {
		   OFF,
		   0x1074,
		   0x1075,
		   0x7CD9,
		   0xF8DA,
		   0x06DD
		 };

/* These are the pointers to specific FIFO tables
   e.g at table dwMPEG1_2MB, 1024x768 has value 0x00020001, which means
   that 8bpp uses wFIFOTable1, 16bpp - wFIFOTable0, 24bpp -- wFIFOTable2,
   FF indicates that the mode is not supported.

	     24 16 8 Bpp
*/
unsigned long dwMPEG1_1MB[] = { 	 /* 1MB at 67 MHz or above MClk */
	  0x00010101,		 /* 640x480 */
	  0x00FFFF01,		 /* 800x600 */
	  0x00FFFF00,		 /*1024x768 */
	  0x00FFFFFF,		 /*1280x1024*/
	  0x00FFFFFF		 /*1600x1200*/
	};

unsigned long dwMPEG1_2MB[] = { 	 /* 2MB/4MB at 67 MHz or above MClk */
	  0x00010101,		 /* 640x480 */
	  0x00000101,		 /* 800x600 */
	  0x00020001,		 /*1024x768 */
	  0x00FFFF00,		 /*1280x1024*/
	  0x00FFFF00		 /*1600x1200*/
	};

unsigned long dwMPEG2_2MB[] = { 	 /* 2MB/4MB at 75 MHz or above MClk */
	  0x00030303,		 /* 640x480 */
	  0x00FFFF03,		 /* 800x600 */
	  0x00FFFF04,		 /*1024x768 */
	  0x00FFFFFF,		 /*1280x1024*/
	  0x00FFFFFF		 /*1600x1200*/
	};

struct	ttype {
	unsigned short  *wFIFOData;
	unsigned short   reserved;
      };


struct	ttype wTable0 = {
	wFIFOTable0,
	0x00
      };

struct	ttype wTable1 = {
	wFIFOTable1,
	0x00
      };

struct	ttype wTable2 = {
	wFIFOTable2,
	0x00
      };

struct	ttype wTable3 = {
	wFIFOTable3,
	0x00
      };

struct	ttype wTable4 = {
	wFIFOTable4,
	0x00
      };

struct	ttype *wFIFOTable[] = {
	&wTable0,
	&wTable1,
	&wTable2,
	&wTable3,
	&wTable4
      };

#endif //_FIFOTAB_H

