/* manipulation of IGS registers */
#include <linux/types.h>
#include <linux/mm.h>

#include "videodev.h"
#include "vidcap.h"
#include "igs.h"
#include "igsvideo.h"
#include "fifotab.h"

/* IGS CyberPro control stuff
 * these first arrays hold the fancy FIFO control bytes
 */
unsigned short iVFilterOn[2] =
{0};
unsigned short iReg74[2] =
{0};
unsigned short iReg75[2] =
{0};
unsigned short iRegD9[2] =
{0};
unsigned short iRegDA[2] =
{0};
unsigned short iRegDD[2] =
{0};

unsigned short savedReg74 = 0;
unsigned short savedReg75 = 0;
unsigned short savedRegD9 = 0;
unsigned short savedRegDA = 0;
unsigned short savedRegDD = 0;

unsigned short wHighResThreshold = 0;
unsigned short wGarbageLine = 0;        /* image padding for half line shift */

#if SUPPORTVUNDERSCAN
int wV_Underscan = 0;
int iMaxLines = 0;				/* # of scan lines within which 
								   * we drop one scan line
								 */
int iWhichLine = 0;				/* which scan line we drop */
#endif

/**********************************************************
 *****************  direct IGS register  routines *********
 **********************************************************/


/* return TRUE if the VGA is in graphics mode */
int
IGS_IsGraphics(void)
{
    /* easy: just check bit 0 of 3cf/6 */
    return In_Video_Reg(6) & 1;
}

/**********************************************************
 IGS_GetCurrentScreen
 pokes around in the vga registers and copies the
 information into dev->screen

 non-invasive
 **********************************************************/
int
IGS_GetCurrentScreen(struct nw_cap_device* pDev)
{
    int i, modeBpp, base, offset;
    unsigned char bCrt01, bCrt12, bCrt07, bExt11;

    /* first, get the bytes_per_pixel */
    outb_p (ExtSeqMisc, IGSEXTINDEX);
    modeBpp = inb_p (IGSEXTDATA) & 0x07;

    switch (modeBpp)
    {
    case 2:
    case 6:
        modeBpp = 2; 	  break;
    case 3:
        modeBpp = 4; 	  break;
    case 4:
        modeBpp = 3; 	  break;
    case 1:
    default:
        modeBpp = 1; 	  break;
    }
    pDev->screen.bytes_per_pixel = modeBpp;

    /* now get the visible screen pixel counts */

    outb_p (0x01, CRTINDEX);
    bCrt01 = inb_p (CRTDATA);
    pDev->screen.width = ((int) bCrt01 + 1) * 8;
    
    outb_p (0x07, CRTINDEX);
    bCrt07 = inb_p (CRTDATA);

    outb_p (0x12, CRTINDEX);
    bCrt12 = inb_p (CRTDATA);

    outb_p (0x11, IGSEXTINDEX);
    bExt11 = inb_p (IGSEXTDATA);

    pDev->screen.height =
        bCrt12 + (((unsigned short) bExt11 & 0x02) << 9)
        + ((bCrt07 & 0x02) << 7) + ((bCrt07 & 0x40) << 3) + 1;
    
    /* get the total x and y size of the pixel map */
    outb_p(0x13,CRTINDEX);
    offset = inb_p(CRTDATA);
    offset += (In_Video_Reg(0x15) & 0x30) << 4;
    pDev->screen.x_ext = (offset * 8) / modeBpp;
    /* we cannot get the y extent from the hardware;
     hopefully the external app will tell us what this is */
    pDev->screen.y_ext = pDev->screen.height;

    /* now get the horz and vert starting pixels */
    outb_p(0x0d, CRTINDEX);
    base = inb_p(CRTDATA);
    outb_p(0x0c, CRTINDEX);
    base += (inb_p(CRTDATA) << 8);
    base += (0x0f & In_Video_Reg(0x10)) << 16;
    
    /* this is the first displayed pixel
     * now break it into x and y
     */
    pDev->screen.y_orig = (4 * base/modeBpp) / pDev->screen.x_ext;
    pDev->screen.x_orig = (4 * base/modeBpp) % pDev->screen.x_ext;

    return 0;
}

#if 0
/**********************************************************
 Get current graphics mode color depth
 in: none
 return 1 - 8 Bpp (256 color mode)
        2 - 16 Bpp (High color mode)
        3 - 24 Bpp (True color mode)
 **********************************************************/
int
IGS_GetBpp (void)
{
  unsigned char modeBpp;

  outb_p (ExtSeqMisc, IGSEXTINDEX);
  modeBpp = inb_p (IGSEXTDATA) & 0x07;

  switch (modeBpp)
	{
	case 2:
	case 6:
	  modeBpp = 2; 	  break;
	case 3:
	  modeBpp = 4; 	  break;
	case 4:
	  modeBpp = 3; 	  break;
	case 1:
	default:
	  modeBpp = 1; 	  break;
	}
  return ((int) modeBpp);
}


/*=============================================
  in:  iScreen is X_SCREEN or Y_SCREEN
  out: return current x_resolution or y_resolution
  The size of the visible screen.
=============================================*/
int
IGS_GetCurrentScreenSize (int iScreen)
{
  int iDE;						/*H or V Display Enable */
  unsigned char bCrt01, bCrt12, bCrt07, bExt11;

  if (iScreen == Y_SCREEN)
	{
	  outb_p (0x07, CRTINDEX);
	  bCrt07 = inb_p (CRTDATA);

	  outb_p (0x12, CRTINDEX);
	  bCrt12 = inb_p (CRTDATA);

	  outb_p (0x11, IGSEXTINDEX);
	  bExt11 = inb_p (IGSEXTDATA);

	  iDE = bCrt12 + (((unsigned short) bExt11 & 0x02) << 9) + ((bCrt07 & 0x02) << 7) + ((bCrt07 & 0x40) << 3) + 1;
	}
  else
	{
	  outb_p (0x01, CRTINDEX);
	  bCrt01 = inb_p (CRTDATA);
	  iDE = ((int) bCrt01 + 1) * 8;
	}

  return iDE;
}
/* get the X or Y starting location for the visible screen
 * NOTE: screen.x_ext and screen.bytes_per_pixel should be
 * set up before this call!
 */
int
IGS_GetCurrentScreenStart(struct nw_vid_device* pDev, int mode)
{
    int base, ret_val;
    outb_p(0x0d, CRTINDEX);
    base = inb_p(CRTDATA);
    outb_p(0x0c, CRTINDEX);
    base += (inb_p(CRTDATA) << 8);
    base += (0x0f & In_Video_Reg(0x10)) << 16;
    
    /* this is the first displayed pixel
     * now break it into x and y
     */
    if (mode == Y_SCREEN)
        ret_val = 2 * base / pDev->screen.x_ext / pDev->screen.bytes_per_pixel;
    else
        ret_val = 2 * base % (pDev->screen.x_ext * pDev->screen.bytes_per_pixel);
#ifdef DEBUG
    printk("vidcap: current %s screen start is %#x\n",
           iScreen==X_SCREEN ? "x" : "y", ret_val);
#endif
    return ret_val;
}
/* Get the X or Y total sizes, which may exceed the
 * visible area
 * The Y value we cannot know for sure, so just return
 * the visible row count.
 * This routine requires screen.height and screen.bytes_per_pixel
 */
int
IGS_GetCurrentScreenExt(struct nw_cap_device* pDev, int mode)
{
    int ret_val;
    int offset;
    outb_p(0x13,CRTINDEX);
    offset = inb_p(CRTDATA);
    offset += (In_Video_Reg(0x15) & 0x30) << 4;
    if (mode == X_SCREEN)
        ret_val = (offset * 8) / pDev->screen.bytes_per_pixel;
    else
        ret_val = screen.height;
#ifdef DEBUG
    printk("vidcap: get current extents returns %d\n",ret_val);
#endif
    return ret_val;
}
#endif


/*=============================================
  in:  None
  out: return current video memory size
       0: 1MB, 1: 2MB, 2: 4MB
=============================================*/
int
IGS_GetVMemSize (void)
{
  unsigned char bData;
  outb_p (0x72, IGSEXTINDEX);
  bData = inb_p (IGSEXTDATA) & 0x03;	/*0: 1MB, 1: 2MB, 2: 4MB. */
  return 0x00100000 << bData;
}


/*=============================================
  This routine is used internally:
  Get Video FIFO and other informations
  return 0: error, 1: success
  requires screen.bytes_per_pixel
=============================================*/
int
GetVideoFIFO (struct nw_cap_device* pDev)
{
  int k, wTableIndex, iMpeg1, iMpeg2;
  unsigned short *wPtr;

  /*save FIFO control regs */
  outb_p (ExtFIFOReg0, IGSEXTINDEX);
  savedReg74 = ((inb_p (IGSEXTDATA)) << 8) + ExtFIFOReg0;

  outb_p (ExtFIFOReg1, IGSEXTINDEX);
  savedReg75 = ((inb_p (IGSEXTDATA)) << 8) + ExtFIFOReg1;

  outb_p (FIFO_TIMING_CTL_L, IGSEXTINDEX);
  savedRegD9 = ((inb_p (IGSEXTDATA)) << 8) + FIFO_TIMING_CTL_L;

  outb_p (FIFO_TIMING_CTL_H, IGSEXTINDEX);
  savedRegDA = ((inb_p (IGSEXTDATA)) << 8) + FIFO_TIMING_CTL_H;

  outb_p (FIFO_CTL_I, IGSEXTINDEX);
  savedRegDD = ((inb_p (IGSEXTDATA)) << 8) + FIFO_CTL_I;

  switch (pDev->screen.width)
	{
	default:
	case 640:
	  wTableIndex = 0;
	  break;

	case 800:
	  wTableIndex = 1;
	  break;

	case 1024:
	  wTableIndex = 2;
	  break;

	case 1280:
	  wTableIndex = 3;
	  break;

	case 1600:
	  wTableIndex = 4;
	  break;
	}

  if (pDev->screen.total_video_memory <= 0x00100000)
	iMpeg1 = (unsigned short) (dwMPEG1_1MB[wTableIndex] >> 8*(pDev->screen.bytes_per_pixel - 1));
  else
	iMpeg1 = (unsigned short) (dwMPEG1_2MB[wTableIndex] >> 8*(pDev->screen.bytes_per_pixel - 1));

  iMpeg1 &= 0x00FF;				/*got the index to the FIFO table (MPEG1) */

  iMpeg2 = (unsigned short) (dwMPEG2_2MB[wTableIndex] >> 8*(pDev->screen.bytes_per_pixel - 1));
  iMpeg2 &= 0x00FF;				/*got the index to the FIFO table (MPEG2) */

  /*Copy data to working sets for MPEG1 and MPEG2 size images */
  wPtr = wFIFOTable[iMpeg1]->wFIFOData;
  iVFilterOn[0] = *wPtr++;
  iReg74[0] = *wPtr++;
  iReg75[0] = *wPtr++;
  iRegD9[0] = *wPtr++;
  iRegDA[0] = *wPtr++;
  iRegDD[0] = *wPtr++;

  if (iMpeg2 != 0xFF)
	{
	  wPtr = wFIFOTable[iMpeg2]->wFIFOData;

	  iVFilterOn[1] = *wPtr++;
	  iReg74[1] = *wPtr++;
	  iReg75[1] = *wPtr++;
	  iRegD9[1] = *wPtr++;
	  iRegDA[1] = *wPtr++;
	  iRegDD[1] = *wPtr++;
	}
  else
	{
	  iVFilterOn[1] = iVFilterOn[0];
	  iReg74[1] = iReg74[0];
	  iReg75[1] = iReg75[0];
	  iRegD9[1] = iRegD9[0];
	  iRegDA[1] = iRegDA[0];
	  iRegDD[1] = iRegDD[0];
	}


  if (iMpeg2 != 0xFF)
	wMPEG2Enable = 1;
  else
	wMPEG2Enable = 0;			/* full screen capture should be disabled */

  if ((iMpeg1 != 0xFF))
	return 1;				/*sucessful */
  else
	return 0;				/*fail (capture can not be supported in this mode) */
}


/**********************************************************
 Initialize IGS Video Hardware and Decoder to a default state
 in:   none

 Note: This function should not be called before DetectDecoder,
       since there is decoder dependant code in this routine
       This is one time only initialization routine.
 **********************************************************/
void
IGS_InitVideo (struct nw_cap_device* pDev)
{

    unsigned long dwL;
#ifdef DEBUG
//    printk("IGS_InitVideo, using camera\n"); // %d\n",WhichCamera);
#endif

#if SUPPORTVUNDERSCAN
    /*get underscan information */
    outb_p(0x45,0x3d4);
  wV_Underscan = (inb_p (0x3d5) & 0x10) >> 4;
  if (wV_Underscan == 1)
  {
      outb_p(0x44,0x3d4);
	  iMaxLines = inb_p (0x3d5) + 0x11;
	  iWhichLine = iMaxLines >> 4;
	  iMaxLines = iMaxLines & 0x0F;
	}
#endif


  /* --- Initialize IGS Video Registers --- */

  Out_Video_Reg (DISP_CTL_I, 0x00);

  /*This is only init to default value before set Reg_E8<2,0>
     final working addr will be set at IGS_SetCaptureAddr */
  dwL = pDev->screen.offset_vidcap_mem >> 2;
  Out_Video_Reg (CAPTURE_ADDR_L, (unsigned char) (dwL & 0x0000ffL));
  Out_Video_Reg (CAPTURE_ADDR_M, (unsigned char) ((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg (CAPTURE_ADDR_H, (unsigned char) ((dwL & 0xff0000L) >> 16));

  /* capture non-interlace? */
  Out_Video_Reg (VFAC_CTL_MODE_III, 0x00);
  /* normal sync polarities */
  Out_Video_Reg (VFAC_CTL_MODE_II, 0x10);

  /* select bank Y */
  outw_p ((0x0100 + CAPTURE_BANK_CTL), IGSEXTINDEX);
  /* [y49-y4a] set up 2nd overlay FIFOs */
  outw_p ((iRegD9[0] & 0xFF00 | V_DST_Y1_LO), IGSEXTINDEX);
  outw_p ((iRegDA[0] & 0xFF00 | V_DST_Y1_HI), IGSEXTINDEX);

  /* [c0-cd] set up extended memory start addresses */
  Out_Video_Reg (MEMORY_START_L, 0x00);
  Out_Video_Reg (MEMORY_START_M, 0x0f);
  Out_Video_Reg (MEMORY_START_H, 0x03);
  Out_Video_Reg (DEST_RECT_TOP_L, 0x00);
  Out_Video_Reg (DEST_RECT_TOP_H, 0x00);
  Out_Video_Reg (DEST_RECT_LEFT_L, 0x20);
  Out_Video_Reg (DEST_RECT_LEFT_H, 0x00);
  Out_Video_Reg (DEST_RECT_RIGHT_L, 0x60);
  Out_Video_Reg (DEST_RECT_RIGHT_H, 0x01);
  Out_Video_Reg (DEST_RECT_BOTTOM_L, 0xe0);
  Out_Video_Reg (DEST_RECT_BOTTOM_H, 0x00);
  Out_Video_Reg (MEMORY_OFFSET_PHASE, 0x50);

  /* [d3-d8] set up extended memory -> screen DDA */
  Out_Video_Reg (DDA_Y_INC_L, 0x00);
  Out_Video_Reg (DDA_Y_INC_H, 0x10);
  Out_Video_Reg (DDA_X_INC_L, 0x00);
  Out_Video_Reg (DDA_X_INC_H, 0x10);
  Out_Video_Reg (DDA_X_INIT_L, 0x0);	/*0x800 */
  Out_Video_Reg (DDA_X_INIT_H, 0x8);
  Out_Video_Reg (DDA_Y_INIT_L, 0x0);	/*0x800 */
  Out_Video_Reg (DDA_Y_INIT_H, 0x8);


  /* [cd-d0] init the color compare regs */
  Out_Video_Reg (COLOR_CMP_RED, 0x00);
  Out_Video_Reg (COLOR_CMP_GREEN, 0x00);
  Out_Video_Reg (COLOR_CMP_BLUE, 0x00);

  /* [db] select YUV 4:2:2 */
  Out_Video_Reg (VIDEO_FORMAT, 0x00);

#if 0
  /* [dc] clear out the coprocessor */
  Out_Video_Reg (MISC_CTL_I, 0x00);
  Out_Video_Reg (MISC_CTL_I, 0x01);
  Out_Video_Reg (MISC_CTL_I, 0xc0);
  Out_Video_Reg (MISC_CTL_I, 0x01);
#endif

  /* [68-6f] set up the camera -> memory DDA */
  outw_p ((0x0000 + CAPTURE_DDA_X_INIT_L), IGSEXTINDEX);
  outw_p ((0x0800 + CAPTURE_DDA_X_INIT_H), IGSEXTINDEX);
  outw_p ((0x0000 + CAPTURE_DDA_Y_INIT_L), IGSEXTINDEX);
  outw_p ((0x0800 + CAPTURE_DDA_Y_INIT_H), IGSEXTINDEX);

  /* disable PIP */
  outw_p ((0x3F00 + CAPTURE_NEW_CTL_I), IGSEXTINDEX);
  /* no interrupts */
  outw_p ((0xC000 + CAPTURE_NEW_CTL_II), IGSEXTINDEX);

  /* 16 bit capture, enable VLF FIFO (?), no mirrors */
  outw_p ((0x1000 + CAPTURE_MODE_I), IGSEXTINDEX);
  /* normal */
  outw_p ((0x5000 + CAPTURE_MODE_II), IGSEXTINDEX);
  /* ?? */
  outw_p ((0x8000 + CAPTURE_TV_CTL), IGSEXTINDEX);
  /* set a flag to indicate that we are capturing */
  outw_p ((0xB400 + CAPTURE_CTL_MISC), IGSEXTINDEX);

  /* select bank K */
  outw_p ((0x0500 + CAPTURE_BANK_CTL), IGSEXTINDEX);
  /* single channel capture, vert interpolate, colorkey,
   video data clipped to 0x10-0xF0, linear uv
   but we are not using x2 so this has no effect */
  outw_p ((0x4000 + CAPTURE_X2_CTL_I), IGSEXTINDEX);

//  switch (WhichCamera)
	{
//	default:
//	case PHILIPS7111:
	  {
		/* check for 8bit video stream from 7111 */
		outb_p (0xbe, IGSEXTINDEX);
		if ((inb_p (IGSEXTDATA) & 0x04) == 0x00)
		  {
			/* set capture mode to 8 bit */
			Out_Video_Reg_M (CAPTURE_MODE_I, 0x01, 0xfe);	/* Enable 8 bit video */

			// Milind:
			outb_p (0xb6, IGSEXTINDEX);

			sendOneByte (P7111Port, 0x10, 0xC8);	/* 7111 - 8 bit */

			/* capture 1408 bytes acros = 704 pixels */
			Out_Video_Reg (CAPTURE_H_START_L, 0x0F);
			Out_Video_Reg (CAPTURE_H_START_H, 0x00);
			Out_Video_Reg (CAPTURE_H_END_L, 0x90);
			Out_Video_Reg (CAPTURE_H_END_H, 0x05);
		  }
		else /* must be 16 bit stream; not a NewWinder! */
		  {
			Out_Video_Reg (CAPTURE_H_START_L, 0x07);
			Out_Video_Reg (CAPTURE_H_START_H, 0x00);
			Out_Video_Reg (CAPTURE_H_END_L, 0xC8);
			Out_Video_Reg (CAPTURE_H_END_H, 0x02);
		  }
	  }
//	  break;

//	case PHILIPS7110:
//	  {
//		Out_Video_Reg (CAPTURE_H_START_L, 0x00);
//		Out_Video_Reg (CAPTURE_H_START_H, 0x00);
//		Out_Video_Reg (CAPTURE_H_END_L, 0x82);
//		Out_Video_Reg (CAPTURE_H_END_H, 0x02);
//	  }
//	  break;
	}

  /* capture off, colorkey, no yuv clipping, uv linear, vert interp on,
   no capture display */
  Out_Video_Reg (DISP_CTL_I, 0x04);

}


/**********************************************************
 Turn on IGS video (Turn on both External Out and External In)
 in:   none
 Note: . This function should be called at least once to enable
	 IGS video hardware.
       . In case video stops accidently, this function together
	 with IGS_VideoOff can be used to reenable the video hardware.
**********************************************************/
void
IGS_CaptureOn (void)
{
#ifdef DEBUG
    printk("IGS_CaptureOn\n");
#endif
  IGS_SetFreezeVid (ON);

  Out_Video_Reg_M (0xb5, 0x01, 0xFE);	/* Enable Vafc */

#if 0
  /* reset the coprocessor */
  Out_Video_Reg (MISC_CTL_I, 0xc0);
  Out_Video_Reg (MISC_CTL_I, 0x01);
#endif

  /* Milind: */
  /* enable capture, synchronous with vga */
  while ((inb_p (0x3da) & 0x08));	/* skip current Vertical Retrace */
  while (!(inb_p (0x3da) & 0x08));	/* wait for Vertical Retrace to begin */

  Out_Video_Reg_M (VFAC_CTL_MODE_I, 0x41, 0xBE);	/*Video capture enable */

  IGS_SetFreezeVid (OFF);
}

/* turn on the capture display */
void IGS_WindowOn(void)
{
#ifdef DEBUG
    printk("IGS_WindowOn\n");
#endif
  /* enable capture display and no yuv range clipping */
  Out_Video_Reg_M (DISP_CTL_I, 0x84, 0x7B);
}

/**********************************************************
 Turn off IGS video (Turn off both External Out and External In)
 in:   none
 **********************************************************/
void
IGS_CaptureOff (void)
{
#ifdef DEBUG
    printk("IGS_CaptureOff\n");
#endif
  /* Milind: */
  while (!(inb_p (0x3da) & 0x08));	/* wait for Vertical Retrace to begin */
  Out_Video_Reg (VFAC_CTL_MODE_I, 0x40);
  Out_Video_Reg_M (0xb5, 0x00, 0xFE);	/* Disable Vafc */

  ResetSeq ();

}
void
IGS_WindowOff(void)
{
#ifdef DEBUG
    printk("IGS_WindowOff\n");
#endif
  Out_Video_Reg (DISP_CTL_I, 0x00);		/* Video disabled */
}


/*=========================================================
 This routine is used internally
 in:   none
 =========================================================*/
void
ResetSeq (void)
{
  unsigned char reg_70;
  outb_p (0x70, IGSEXTINDEX);
  reg_70 = inb_p (IGSEXTDATA);
  outb_p (reg_70 | 0x80, IGSEXTDATA);
  outb_p (reg_70 & 0x7f, IGSEXTDATA);
}


/*=========================================================
 This routine is used internally
 in:   none

  Byte OUTPORT with bit mask
  in: bValue=bit/bits to be set/clear, untouched bit/bits must be zero
      bMask=bit/bits to be masked, 1:mask, 0:to be set/clear
  e.g. set <4,3>=<1,0>, bValue=0x10 and bMask=E7
 =========================================================*/
void
Out_Video_Reg_M (unsigned char bIndex, unsigned char bValue, unsigned char bMask)
{
  outb_p (bIndex, IGSEXTINDEX);
  outb_p ((inb_p (IGSEXTDATA) & bMask) | bValue, IGSEXTDATA);
}


/*=========================================================
 This routine is used internally
 in:   none
 =========================================================*/
void
Out_Video_Reg (unsigned char bIndex, unsigned char bValue)
{
  outb_p (bIndex, IGSEXTINDEX);
  outb_p (bValue, IGSEXTDATA);

}

unsigned char
In_Video_Reg (unsigned char bIndex) {
    outb_p(bIndex, IGSEXTINDEX);
    return inb_p(IGSEXTDATA);
}






/**********************************************************
 Setup captured video format
 in:  nFormat = YUV422
	     else: not support yet
 **********************************************************/
void
IGS_SetCaptureFormat (int nFormat)
{
  switch (nFormat)
	{
	case PIX_FMT_YUYV:
	  Out_Video_Reg_M (VIDEO_FORMAT, 0x00, 0xF8);
	  break;

	default:
	  break;
        }
#ifdef DEBUG
  printk("SetCaptureFormat %.4s\n",&nFormat);
#endif
}


/**********************************************************
 Setup video overlay format
 in:  nFormat = COLORKEY
	      = WINDOWKEY
 **********************************************************/
void
IGS_SetOverlayFormat (unsigned char nFormat)
{
  switch (nFormat)
	{
	case COLORKEY:
	  Out_Video_Reg_M (DISP_CTL_I, 0x00, 0xFD);
	  break;

	case WINDOWKEY:
	default:
	  Out_Video_Reg_M (DISP_CTL_I, 0x02, 0xFD);
	  break;
	}
}


/**********************************************************
 Setup video 2 overlay format
 in:  nFormat = COLORKEY
	      = WINDOWKEY
 **********************************************************/
void
IGS_SetOverlayFormat_V2 (unsigned char nFormat)
{
  Out_Video_Reg (CAPTURE_BANK_CTL, 0x01);
  switch (nFormat)
	{
	case COLORKEY:
	  Out_Video_Reg_M (V_DST_Y2_HI, 0x00, 0xFD);
	  break;

	case WINDOWKEY:
	default:
	  Out_Video_Reg_M (V_DST_Y2_HI, 0x02, 0xFD);
	  break;
	}
}

/* set the capture DDA
 * take advantage of small captures to turn off the interlace
 * set capture clipping
 */
void
IGS_SetCaptureScaling(struct nw_cap_device* pDev)
{
#ifdef DEBUG
    printk("IGS_SetCaptureScaling %d %d\n",
           pDev->capture_format.width,
           pDev->capture_format.height);
#endif
  if (pDev->capture_format.width * 2 <= wHighResThreshold)
	Change_VideoFIFO (0);		/*set MPEG1 FIFO */
  else
	Change_VideoFIFO (1);		/*set MPEG2 FIFO */

  if (pDev->capture_format.height <= 240)
	{
	  if (pDev->videoStandard == NTSC)
		{
                    Set_Capture_Scale_Registers (pDev->camera_data_size.width,
                                       pDev->capture_format.width,
                                       NTSC_SRC_HEIGHT / 2,
                                       pDev->capture_format.height);
		  wGarbageLine = NTSC_GarbageLine / 2;
		}
	  else if (pDev->videoStandard == PAL)
		{
                    Set_Capture_Scale_Registers (pDev->camera_data_size.width,
                                       pDev->capture_format.width,
                                       PAL_SRC_HEIGHT / 2,
                                       pDev->capture_format.height);
		  wGarbageLine = PAL_GarbageLine / 2;
		}

          /* capture only half the lines; twice as fast!
             and avoids motion blur */
	  Out_Video_Reg_M (VFAC_CTL_MODE_III, 0x00, 0xFD);	/*Interlace bit=0 */
	  Out_Video_Reg_M (CAPTURE_CTL_MISC, 0x04, 0xFB);	/*Data Good bit=1 */
	}
  else
	{
	  if (pDev->videoStandard == NTSC)
		{
                    Set_Capture_Scale_Registers (pDev->camera_data_size.width,
                                       pDev->capture_format.width,
                                       NTSC_SRC_HEIGHT,
                                       pDev->capture_format.height);
		  wGarbageLine = NTSC_GarbageLine;
		}
	  else if (pDev->videoStandard == PAL)
		{
                    Set_Capture_Scale_Registers (pDev->camera_data_size.width,
                                       pDev->capture_format.width,
                                       PAL_SRC_HEIGHT,
                                       pDev->capture_format.height);
		  wGarbageLine = PAL_GarbageLine;
		}

	  Out_Video_Reg_M (VFAC_CTL_MODE_III, 0x02, 0xFD);	/*Interlace bit=1 */
	  Out_Video_Reg_M (CAPTURE_CTL_MISC, 0x00, 0xFB);	/*Data Good bit=0 */
	}

  /* set the capture pitch */
  Out_Video_Reg (CAPTURE_PITCH_L, (unsigned char) ((pDev->capture_format.width * 2) >> 3));

  IGS_SetCaptureClip (pDev->videoStandard);
}

/**********************************************************
 Setup capture window address (Setup incoming video address)
 The address depends on whether the image is mirrored, so
 let the mirror routine set the hardware start address
 **********************************************************/
void
IGS_SetCaptureAddr (struct nw_cap_device* pDev,
                    int offset)
{
  /* work out mirrors and set the hardware capt address register */
    IGS_SetXYMirror (pDev, offset);

}


/**********************************************************
 Scale up the overlay window (source window unchange)
 in:
     wSrcXExt - source window width
     wDstXExt - destination window width
     wSrcYExt - source window height
     wDstYExt - destination window height
 **********************************************************/
void
IGS_SetWindowScale (unsigned short wSrcXExt, unsigned short wDstXExt, unsigned short wSrcYExt, unsigned short wDstYExt)
{
  unsigned long dwScale, d;

  if (wSrcXExt == wDstXExt)
	dwScale = 4096;				/*0x1000 */
  else
	dwScale = ((unsigned long) (wSrcXExt - 2) * 4096) / (unsigned long) wDstXExt;

  Out_Video_Reg (DDA_X_INC_L, LOBYTE ((unsigned short) dwScale));
  Out_Video_Reg (DDA_X_INC_H, HIBYTE ((unsigned short) dwScale));

  // delete me !!
  d = dwScale;
  
  if (wSrcYExt == wDstYExt)
	dwScale = 4096;				/*0x1000 */
  else
	dwScale = ((unsigned long) (wSrcYExt - 2) * 4096) / (unsigned long) wDstYExt;

  Out_Video_Reg (DDA_Y_INC_L, LOBYTE ((unsigned short) dwScale));
  Out_Video_Reg (DDA_Y_INC_H, HIBYTE ((unsigned short) dwScale));

#ifdef DEBUG
  printk("vidcap: window scale using %d %d %d %d: coef: %x %x\n",wSrcXExt,wDstXExt,
         wSrcYExt, wDstYExt, d, dwScale);
#endif
}


/**********************************************************
 Scale up video 2 overlay window (video 2 source window unchange)
 in:
     wSrcXExt - source window width
     wDstXExt - destination window width
     wSrcYExt - source window height
     wDstYExt - destination window height
 **********************************************************/
void
IGS_SetScale_V2 (unsigned short wSrcXExt, unsigned short wDstXExt, unsigned short wSrcYExt, unsigned short wDstYExt)
{
  unsigned long dwScale;

  Out_Video_Reg (CAPTURE_BANK_CTL, 0x01);

  /*initial value registers are same as video 1 */
  Out_Video_Reg (DDA_X_INIT_L, 0x0);
  Out_Video_Reg (DDA_X_INIT_H, 0x8);

  if (wSrcXExt == wDstXExt)
	dwScale = 4096;
  else
	dwScale = ((unsigned long) (wSrcXExt - 2) * 4096) / (unsigned long) wDstXExt;

  Out_Video_Reg (V_SRC_STRIDE_LO, LOBYTE ((unsigned short) dwScale));
  Out_Video_Reg (V_SRC_STRIDE_HI, HIBYTE ((unsigned short) dwScale));

  /*initial value registers are same as video 1 */
  Out_Video_Reg (DDA_Y_INIT_L, 0x0);
  Out_Video_Reg (DDA_Y_INIT_H, 0x8);

  if (wSrcYExt == wDstYExt)
	dwScale = 4096;
  else
	dwScale = ((unsigned long) (wSrcYExt - 2) * 4096) / (unsigned long) wDstYExt;

  Out_Video_Reg (V_DST_X2_LO, LOBYTE ((unsigned short) dwScale));
  Out_Video_Reg (V_DST_X2_HI, HIBYTE ((unsigned short) dwScale));

  Out_Video_Reg (CAPTURE_BANK_CTL, 0x00);
}


/**********************************************************
 Create and Position the overlay window with adjustment
 if overlay windows is moved to far left or far right
 in:  Rectangle to be overlayed to the screen:
	cx  - current x of the window
	cy  - current y of the window
	wWinWidth  - current window width
        wWinHeight - current window height
 Reconcile a window display request (state.requested_window),
 given in logical coordinates, with a currently displayed
 screen (screen structure). The current screen may be only
 parially displayed, so the result of this calculation is put
 back into state.realized_window.

 Then set the hardware with the resulting data.
 
 given: state.requested_window
        screen.x_ext, .y_ext, .x_orig, .y_orig, .width, .height
 return: state.realized_window
 **********************************************************/
void
IGS_AdjustWindow (struct nw_cap_device* pDev)
{
  int i = 0, j = 0, wX = 0, wY = 0, iDstWidth = 0;

  /* check left limit */
  pDev->realized_window.x = max(pDev->requested_window.x - pDev->screen.x_orig,0);
  /* check right limit */
  j = min(pDev->requested_window.x + pDev->requested_window.width,
          pDev->screen.x_orig + pDev->screen.width);
#if 0
  printk("IGS_AdjustWindow: set realized width to %d, which is based on "
         "requested x %d, requested width %d, screen orig %d, screen width %d\n",
         j - pDev->realized_window.x, pDev->requested_window.x, pDev->requested_window.width,
         pDev->screen.x_orig, pDev->screen.width);
#endif
  pDev->realized_window.width = j - pDev->realized_window.x - pDev->screen.x_orig;

  /* check top limit */
  pDev->realized_window.y = max(pDev->requested_window.y - pDev->screen.y_orig,0);
  /* check bottom limit */
  i = min(pDev->requested_window.y + pDev->requested_window.height,
          pDev->screen.y_orig + pDev->screen.height);
  pDev->realized_window.height = i - pDev->realized_window.y - pDev->screen.y_orig;

  /* now set the scaling for capture->display */
  /* !! not yet done: if IGS_AdjustWindow puts us up against
   an edge, then fetch offset has to change ! */
  IGS_SetWindowScale(pDev->capture_format.width,
                     pDev->requested_window.width,
                     pDev->capture_format.height,
                     pDev->requested_window.height);

  IGS_SetSrcAddr(pDev,
                 0, //pDev->realized_window.x,
                 0, //pDev->realized_window.y,
                 pDev->capture_format.width, // pDev->realized_window.width,
                 pDev->capture_format.width); //pDev->realized_window.width);

  /* clean this up; no need for args1-5 !! */
  IGS_SetOverlayWindow (pDev,
                        pDev->realized_window.x,
                        pDev->realized_window.y,
                        pDev->realized_window.x + pDev->realized_window.width,
                        pDev->realized_window.y + pDev->realized_window.height);

#ifdef DEBUG
  printk("vidcap: Adjust window called, setting top/left to %d/%d and w/h to %d/%d\n",
         pDev->realized_window.x, pDev->realized_window.y,
         pDev->realized_window.width, pDev->realized_window.height);
#endif
}




/**********************************************************
 Create and Position the 2nd overlay window with adjustment
 if overlay windows is moved to far left or far right
 in:  Rectangle to be overlayed to the screen:
	cx  - current x of the window
	cy  - current y of the window
	wWinWidth  - current window width
	wWinHeight - current window height
 **********************************************************/
void
IGS_AdjustVideoWindow_V2 (int cx, int cy, int wWinWidth, int wWinHeight)
{
#if 0
  int i, j, wX, wY;

  if (cx > 0)
	{
	  wX = 0;
	  i = cx;
	}
  else
	{
	  i = 0;
	  wX = abs (cx);
	  wX = (unsigned short) (((unsigned long) wX * wDEFAULT_SRC_WIDTH_V2) / (unsigned long) wWinWidth);
	}

  if (cy > 0)
	{
	  wY = 0;
	  j = cy;
	}
  else
	{
	  j = 0;
	  wY = abs (cy);
	  wY = (unsigned short) (((unsigned long) wY * wDEFAULT_SRC_HEIGHT_V2) / (unsigned long) wWinHeight);
	}

  /*The following line will support Video Screen being moved to far left */
  IGS_SetSrcAddr_V2 (wX, wY, wDEFAULT_SRC_WIDTH_V2, wDEFAULT_SRC_WIDTH_V2);
  IGS_SetOverlayWindow_V2 (i, j, cx + wWinWidth, cy + wWinHeight);
#endif
}


/*=============================================
  This routine is used internally:
  Routine: to change Video FIFO
  in: which=table to be used
=============================================*/
void
Change_VideoFIFO (int which)
{
  if (iVFilterOn[which] == 0)
	Out_Video_Reg_M (DISP_CTL_I, 0x20, 0xDF);
  else
	Out_Video_Reg_M (DISP_CTL_I, 0x00, 0xDF);

  outw_p (iReg74[which], IGSEXTINDEX);
  outw_p (iReg75[which], IGSEXTINDEX);
  outw_p (iRegD9[which], IGSEXTINDEX);
  outw_p (iRegDA[which], IGSEXTINDEX);
  outw_p (iRegDD[which], IGSEXTINDEX);
#ifdef DEBUG
  printk("vidcap: fifo registers changed to %s\n",which ? "mpeg1" : "mpeg2");
#endif
}


/*=============================================
 This routine is used internally
 in: source X,Y is the camera data: usually 640/720 by 480/240
     dest: capture buffer size
===============================================*/
void
Set_Capture_Scale_Registers (unsigned short wSrcXExt,
                   unsigned short wDstXExt,
                   unsigned short wSrcYExt,
                   unsigned short wDstYExt)
{
  unsigned long dwScale;

#ifdef DEBUG
  printk("Set_Capture_Scale_Registers %d %d %d %d\n",wSrcXExt,
         wDstXExt, wSrcYExt, wDstYExt);
#endif
  if (wSrcXExt == wDstXExt)
	dwScale = 4096;
  else
        dwScale = ((unsigned long) (wDstXExt) * 4096) / (unsigned long) wSrcXExt;

  Out_Video_Reg (CAPTURE_DDA_X_INC_L, LOBYTE ((unsigned short) dwScale));
  Out_Video_Reg (CAPTURE_DDA_X_INC_H, HIBYTE ((unsigned short) dwScale));

  if (wSrcYExt == wDstYExt)
	dwScale = 4096;
  else
	  dwScale = (((unsigned long) wDstYExt * 4096) / (unsigned long) wSrcYExt) + 0x1E;

  Out_Video_Reg (CAPTURE_DDA_Y_INC_L, LOBYTE ((unsigned short) dwScale));
  Out_Video_Reg (CAPTURE_DDA_Y_INC_H, HIBYTE ((unsigned short) dwScale));

}


/*=============================================
  This routine is used to set Capture Src Clip
  in:
    gwVideoStandard == NTSC or PAL
=============================================*/
void
IGS_SetCaptureClip (int gwVideoStandard)
{
//  switch (WhichCamera)
//	{
//	default:
//	case PHILIPS7111:
	  if (gwVideoStandard == NTSC)
		{
		  outw_p ((0x0B00 + CAPTURE_V_START_L), IGSEXTINDEX);	/*some garbage lines */
		  outw_p ((0x0000 + CAPTURE_V_START_H), IGSEXTINDEX);
		  outw_p ((0xFD00 + CAPTURE_V_END_L), IGSEXTINDEX);     /*480+GarbageLines=end-start */
		  outw_p ((0x0000 + CAPTURE_V_END_H), IGSEXTINDEX);
		}
	  else if (gwVideoStandard == PAL)
		{
		  outw_p (0x1000 + CAPTURE_V_START_L, IGSEXTINDEX);	/*some garbage lin */
		  outw_p (0x0000 + CAPTURE_V_START_H, IGSEXTINDEX);
		  outw_p (0x3200 + CAPTURE_V_END_L, IGSEXTINDEX);	/*576+GarbageLines=end-start */
		  outw_p (0x0100 + CAPTURE_V_END_H, IGSEXTINDEX);
		}
//	  break;

//	case PHILIPS7110:
//	  if (gwVideoStandard == NTSC)
//		{
//		  outw_p (0x0B00 + CAPTURE_V_START_L, IGSEXTINDEX);
//		  outw_p (0x0000 + CAPTURE_V_START_H, IGSEXTINDEX);
//		  outw_p (0xFD00 + CAPTURE_V_END_L, IGSEXTINDEX);
//		  outw_p (0x0000 + CAPTURE_V_END_H, IGSEXTINDEX);
//		}
//	  else if (gwVideoStandard == PAL)
//		{
//		  outw_p (0x1000 + CAPTURE_V_START_L, IGSEXTINDEX);
//		  outw_p (0x0000 + CAPTURE_V_START_H, IGSEXTINDEX);
//		  outw_p (0x3200 + CAPTURE_V_END_L, IGSEXTINDEX);
//		  outw_p (0x0100 + CAPTURE_V_END_H, IGSEXTINDEX);
//		}
//	  break;
//	}

}


/**********************************************************
  To mirror capture image
  in:
  gwXMirror: 1: x-mirror, 0: x-normal
  gwYMirror: 1: y-mirror, 0: y-normal
 **********************************************************/
void
IGS_SetXYMirror (struct nw_cap_device* pDev,
                 int offset)
{
  unsigned long dwL;

  dwL = offset;

  if (pDev->xMirrorDisplay)
	{
            dwL += (unsigned long) (pDev->capture_format.height + 2)
                * pDev->capture_format.width * 2;
	  Out_Video_Reg_M (CAPTURE_MODE_I, 0x40, 0xBF);		/*Y mirror */
	  Out_Video_Reg_M (VFAC_CTL_MODE_II, 0x20, 0xDF);	/*O-E polarity reverse */
	}
  else
	{
	  Out_Video_Reg_M (CAPTURE_MODE_I, 0x00, 0xBF);		/*Y normal */
	  Out_Video_Reg_M (VFAC_CTL_MODE_II, 0x00, 0xDF);	/*O-E polarity normal */
	}

  if (pDev->yMirrorDisplay)
	{
	  dwL += (unsigned long) pDev->capture_format.width * 2 - 2;
	  Out_Video_Reg_M (CAPTURE_MODE_I, 0x80, 0x7F);		/*X mirror */
	}
  else
	Out_Video_Reg_M (CAPTURE_MODE_I, 0x00, 0x7F);	/*X normal */

  IGS_SetFreezeVid (ON);		/*only need freeze when set capture addr */

  Set_Capture_addr (dwL);

  IGS_SetFreezeVid (OFF);
}


/*=========================================================
 This routine is used internally
 in:   OnOff = ON or OFF
 Note: This routine is used only at
	 1) anytime when we touch capture address registers (0xEB,0xEC,0xED)
	 2) anytime when we touch video capture enable register (0xE8)
 =========================================================*/
void
IGS_SetFreezeVid (int OnOff)
{
  if (OnOff == ON)				/*note Freeze is On (i.e capture is off) */
  {
      /* turn on immediate freeze and sync freeze bits */
      Out_Video_Reg_M (VFAC_CTL_MODE_I, 0x0C, 0xF3);
      // alternate freeze
      //Out_Video_Reg_M(CAPTURE_MODE_II, 0x80, 0x7F);
      /* wait for freeze status */
      outb_p (VIDEO_ROM_UCB4G_HI, IGSEXTINDEX);
      while (!(inb_p (IGSEXTDATA) & 0x02));
  }
  else
  {
      /* sync with vga; without this, an un-freeze right after
       a capture-enable seems to scribble into undefined vga-ram addresses,
       as if the capture process is already started and cannot be stopped.
       With this delay, the scribbling seems to stop */
      while ((inb_p (0x3da) & 0x08));
      while (!(inb_p (0x3da) & 0x08));
      /* turn off the immediate and sync freeze bits */
      Out_Video_Reg_M (VFAC_CTL_MODE_I, 0x00, 0xF3);
      // alternate unfreeze
      //Out_Video_Reg_M(CAPTURE_MODE_II, 0x00, 0x7F);
  }

  ResetSeq ();
#ifdef DEBUG
//  printk("vidcap: freeze turned %s\n",OnOff ? "on" : "off");
#endif
}
void
IGS_SetFreezeVidAsync (int OnOff)
{
  if (OnOff == ON)				/*note Freeze is On (i.e capture is off) */
  {
      /* turn on sync freeze bits */
//      Out_Video_Reg_M (CAPTURE_MODE_II, 0x80, 0x7f);
      Out_Video_Reg_M (VFAC_CTL_MODE_I, 0x08, 0xF7);
  }
  else
  {
      /* turn them both off */
      Out_Video_Reg_M (VFAC_CTL_MODE_I, 0x00, 0xF3);
//      Out_Video_Reg_M (CAPTURE_MODE_II, 0x00, 0x7f);
  }

//  ResetSeq ();
#ifdef DEBUG
//  printk("vidcap: async freeze turned %s\n",OnOff ? "on" : "off");
#endif
}

int
IGS_IsFreezeDone(void) {
    outb_p (VIDEO_ROM_UCB4G_HI, IGSEXTINDEX);
    return (inb_p (IGSEXTDATA) & 0x02);
}

/*=============================================
  This routine is used internally:
  To set Capture Address
  in: dwOffAddr points to off-screen address with garbage line adjustment
  Note: Freeze and Unfreeze should be used before and after this function.
=============================================*/
void
Set_Capture_addr (unsigned long dwOffAddr)
{
  unsigned long dwL;

  dwL = dwOffAddr >> 2;
  Out_Video_Reg (CAPTURE_ADDR_L, (unsigned char) (dwL & 0x0000ffL));
  Out_Video_Reg (CAPTURE_ADDR_M, (unsigned char) ((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg (CAPTURE_ADDR_H, (unsigned char) ((dwL & 0xff0000L) >> 16));
#ifdef DEBUG
  printk("Set_Capture_addr sets hardware reg to %p (actually %p)\n", dwOffAddr, dwL);
#endif
}


/**********************************************************
 Clean up some status which were set during the video overlay/capture
 in:   none

 Note: This function should be called just before the application exits
 **********************************************************/
void
IGS_VideoCleanUp (void)
{
  /*restore FIFO control regs */
  outw_p (savedReg74, IGSEXTINDEX);
  outw_p (savedReg75, IGSEXTINDEX);
  outw_p (savedRegD9, IGSEXTINDEX);
  outw_p (savedRegDA, IGSEXTINDEX);
}

/**********************************************************
 Clean up some status for video 2 overlay hardware
 in:   none

 Note: This function should be called right after application
       closes video 2.
 **********************************************************/
void
IGS_VideoCleanUp_V2 (void)
{
  int i;
  int iStart = 0x40;
  int iMaxNum = 0x4D;

  /*clear Video_2 registers */
  Out_Video_Reg (CAPTURE_BANK_CTL, 0x00);
  for (i = iStart; i < iMaxNum; i++)
	Out_Video_Reg ((unsigned char) i, 0x00);
}


/**********************************************************
 Trun off video 2 overlay
 in:   none
 **********************************************************/
void
IGS_OverlayOff_V2 (void)
{
  Out_Video_Reg (CAPTURE_BANK_CTL, 0x01);
  Out_Video_Reg_M (0x4D, 0x00, 0x7F);
}


/**********************************************************
 Trun on video 2 overlay
 in:   none
 **********************************************************/
void
IGS_OverlayOn_V2 (void)
{
  Out_Video_Reg (CAPTURE_BANK_CTL, 0x01);
  Out_Video_Reg_M (0x4D, 0x80, 0x7F);
}


/**********************************************************
 Setup color key registers
 in: cref - 32 bit values for rgb color, R,G,B,x with R in lower byte

 Note: . This function is only useful when color key format is used.
       . At 256 color mode, inb_put data should be R=G=B.
       . At 16 Bpp mode, inb_put data should be RGB = RGB & 0xF8FCF8
	    which is RGB565 format.
 **********************************************************/
void
IGS_SetRGBKeyColor (unsigned long cref)
{
  unsigned char bData;
#ifdef DEBUG
  printk("vidcap: chromakey registers set to %p\n",cref);
#endif
  bData = (unsigned char) cref;
  Out_Video_Reg (COLOR_CMP_RED, bData);
  bData = (unsigned char) (cref >> 8);
  Out_Video_Reg (COLOR_CMP_GREEN, bData);
  bData = (unsigned char) (cref >> 16);
  Out_Video_Reg (COLOR_CMP_BLUE, bData);
}


/**********************************************************
 Setup video overlay window (destination window)
 in:  Rectangle to be overlayed to the screen:
	wLeft	(x1)
	wTop	(y1)
	wRight	(x2)
	wBottom (y2)
 **********************************************************/
void
IGS_SetOverlayWindow (struct nw_cap_device* pDev,
                      unsigned short wLeft,
                      unsigned short wTop,
                      unsigned short wRight,
                      unsigned short wBottom)
{
  int iDropLine, iRemainder;

  /*cap the right screen to be the screen width */
  if (wRight > (unsigned short) pDev->screen.x_orig + pDev->screen.width)
	wRight = pDev->screen.x_orig + pDev->screen.width;

#if SUPPORTVUNDERSCAN
  if (wV_Underscan == 1)
	{
	  iDropLine = wTop / iMaxLines;
	  iRemainder = wTop % iMaxLines;

	  if (iRemainder > 0)
		iDropLine++;

	  wTop = wTop - iDropLine;
	}
#endif

  Out_Video_Reg (DEST_RECT_LEFT_L, LOBYTE (wLeft));
  Out_Video_Reg (DEST_RECT_LEFT_H, HIBYTE (wLeft));
  Out_Video_Reg (DEST_RECT_RIGHT_L, LOBYTE (wRight));
  Out_Video_Reg (DEST_RECT_RIGHT_H, HIBYTE (wRight));

  Out_Video_Reg (DEST_RECT_TOP_L, LOBYTE (wTop));
  Out_Video_Reg (DEST_RECT_TOP_H, HIBYTE (wTop));
  Out_Video_Reg (DEST_RECT_BOTTOM_L, LOBYTE (wBottom));
  Out_Video_Reg (DEST_RECT_BOTTOM_H, HIBYTE (wBottom));
}



/**********************************************************
 Setup source video window address (source window to be overlayed)
 in:
     wX 	 - source start x
     wY 	 - source start y
     wPixelWidth - source window width (similar to XGA dimension 1)
     wMapWidth	 - source window map width

     Note: Source height is not specifically defined and it is embedded
	   in destination window and DDA parameters (in IGS_SetScale)
 **********************************************************/
void
IGS_SetSrcAddr (struct nw_cap_device* pDev,
                unsigned short wX, unsigned short wY,
                unsigned short wPixelWidth, unsigned short wMapWidth)
{
  unsigned long dwL;
  unsigned char bHigh;
  unsigned short wFetchPitch;
  unsigned short wBytePitch = wMapWidth * 2;

  /*Reduce some scanlines to avoid garbages from decoder */
  dwL = (pDev->screen.offset_vidcap_mem + (wGarbageLine + wY) * wBytePitch + wX * 2) >> 2;

  /*playback start addr */
  Out_Video_Reg (MEMORY_START_L, (unsigned char) (dwL & 0x0000ffL));
  Out_Video_Reg (MEMORY_START_M, (unsigned char) ((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg (MEMORY_START_H, (unsigned char) ((dwL & 0xff0000L) >> 16));

  /*pitch is a multiple of 64 bits */
  wBytePitch = wBytePitch >> 3;
  wFetchPitch = (wPixelWidth * 2 + 7) >> 3;

  bHigh = HIBYTE (wBytePitch);
  bHigh = bHigh | ((HIBYTE (wFetchPitch)) << 4);

  Out_Video_Reg (MEMORY_PITCH_L, LOBYTE (wBytePitch));
  Out_Video_Reg (MEMORY_PITCH_H, bHigh);

  Out_Video_Reg (MEMORY_OFFSET_PHASE, LOBYTE (wFetchPitch));
#ifdef DEBUG
  printk("vidcap: set capture->window params using %d %d %d %d\n",
         wX, wY, wPixelWidth, wMapWidth);
//  printk("vidcap: set mem pitch to %d and offset phase to %d\n",
//         wBytePitch, wFetchPitch);
#endif
}


/**********************************************************
 Setup source video 2 window address (source window to be overlayed)
 in:
     wX 	 - source start x
     wY 	 - source start y
     wPixelWidth - source window width (similar to XGA dimension 1)
     wMapWidth	 - source window map width

     Note: Source height is not specifically defined and it is embedded
	   in destination window and DDA parameters (in IGS_SetScale)
 **********************************************************/
void
IGS_SetSrcAddr_V2 (unsigned short wX, unsigned short wY, unsigned short wPixelWidth, unsigned short wMapWidth)
{
#if 0
  unsigned long dwL;
  unsigned char bHigh;
  unsigned short wFetchPitch;
  unsigned short wBytePitch = wMapWidth * 2;

  /*get to 2nd video offset */
  dwL = (dwOffsetOffScreen_V2 + wY * wBytePitch + wX * WBPP) >> 2;

  /*playback start addr */
  Out_Video_Reg (CAPTURE_BANK_CTL, 0x00);

  Out_Video_Reg (V_SRC_START_LO, (unsigned char) (dwL & 0x0000ffL));
  Out_Video_Reg (V_SRC_START_MI, (unsigned char) ((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg (V_SRC_START_HI, (unsigned char) ((dwL & 0xff0000L) >> 16));

  /*pitch is a multiple of 64 bits */
  wBytePitch = wBytePitch >> 3;	/*64 bit address field */
  wFetchPitch = (wPixelWidth * WBPP + 7) >> 3;

  bHigh = HIBYTE (wBytePitch);
  bHigh = bHigh | ((HIBYTE (wFetchPitch)) << 4);

  Out_Video_Reg (V_SRC_STRIDE_LO, LOBYTE (wBytePitch));
  Out_Video_Reg (V_SRC_STRIDE_HI, bHigh);

  Out_Video_Reg (V_SRC_FETCH_LO, LOBYTE (wFetchPitch));
#endif
}

/**********************************************************
 Set Picture in Picture window
  in:   x0, y0, x1, y1 are (x,y) position relative to captured
	      data area (not screen coordination).

	iCaptureW, iCaptureH are capture width and height.

	E.g. if capture size is 352x240, then (x0,y0) and (x1,y1)
	are relative to the area (0,0) and (351,239).
	iCaptureW, iCaptureH are 352 and 240.

**********************************************************/
#if 0
void
IGS_SetPIPWindow (int x0, int y0, int x1, int y1, int iCaptureW, int iCaptureH)
{
  unsigned long wX0, wX1;
  unsigned long wY0, wY1;

  unsigned short wReg6061;
  unsigned short wReg6465;

  wReg6061 = In_Video_Reg (CAPTURE_H_START_H);
  wReg6061 = (wReg6061 << 8) + In_Video_Reg (CAPTURE_H_START_L);
  wReg6465 = In_Video_Reg (CAPTURE_V_START_H);
  wReg6465 = (wReg6465 << 8) + In_Video_Reg (CAPTURE_H_START_L);

  wX0 = (unsigned long) 2 *x0;
  wX0 = (unsigned long) wX0 * camera_data_size.width / iCaptureW;
  wX0 += wReg6061;				/*add capture clip information */

  wX1 = (unsigned long) 2 *x1;
  wX1 = (unsigned long) wX1 *camera_data_size.width / iCaptureW;
  wX1 += wReg6061;				/*add capture clip information */

  wY0 = (unsigned long) y0 *camera_data_size.height / iCaptureH;
  wY0 += wReg6465;				/*add capture clip information */

  wY1 = (unsigned long) y1 *camera_data_size.height / iCaptureH;
  wY1 += wReg6465;				/*add capture clip information */

  Out_Video_Reg (CAP_PIP_HSTART_L, (unsigned char) wX0);
  Out_Video_Reg (CAP_PIP_HSTART_H, (unsigned char) (wX0 >> 8));
  Out_Video_Reg (CAP_PIP_HEND_L, (unsigned char) wX1);
  Out_Video_Reg (CAP_PIP_HEND_H, (unsigned char) (wX1 >> 8));

  Out_Video_Reg (CAP_PIP_VSTART_L, (unsigned char) wY0);
  Out_Video_Reg (CAP_PIP_VSTART_H, (unsigned char) (wY0 >> 8));
  Out_Video_Reg (CAP_PIP_VEND_L, (unsigned char) wY1);
  Out_Video_Reg (CAP_PIP_VEND_H, (unsigned char) (wY1 >> 8));
}
#endif

/**********************************************************
	Enable/Disable Picture in Picture
	in:   ON  - enable PiP
	OFF - disable PiP
**********************************************************/
void
IGS_EnablePIP (int OnOff)
{
  if (OnOff == ON)
	Out_Video_Reg_M (CAPTURE_NEW_CTL_I, 0x80, 0x7F);	//enable PIP

  else
	Out_Video_Reg_M (CAPTURE_NEW_CTL_I, 0x00, 0x7F);	//disable PIP

}
#if 0 /* not needed any more */
/*-----------------------------------------------------
  Enable Graphics Engine
  This routine shows an example that enables graphics engine's memory
  mapped IO at base of 0xBF000 (notice this is working DOS program).
  However, enable to linear address is recommanded.
  E.g. video memory is located at 0xFE000000, we can use
       0xFE8BF000 as graphics engine's memory mapped IO base,
       We need set 3CE_33 bits 0,1,2,3, that is,
	 OUTPORT(igs_index_port, 0x0F33);
  -----------------------------------------------------*/
void
EnableCop ()
{
  // orig : outw_p(0x1C33, IGSEXTINDEX);    //enable graphics engine at BF000
  outw_p (0x0f33, IGSEXTINDEX);	//enable graphics engine at BF000

}


/**********************************************************
 Set pointer to video memory used to store captured data

 in: dwL - 32 bit value pointing to video memory, usually is
     off-screen to store YUV422 captured data.

 Note:
       . dwL must be physical memory pointer, not virtual.
	 Please see Init_ConfigData as an example
       . With this pointer, application can access captured
	 image data directly.
 **********************************************************/
void
dwSetOffScreenPtr (unsigned long dwL)
{
    dwOffsetOffScreen = dwL;
    pOffScreen = (unsigned char*)dwL + VGA_IO_ADDR;
}


/**********************************************************
 Set pointer to video memory used by Video 2 overlay window

 in: dwL - 32 bit value pointing to video memory, usually is
     off-screen to store YUV422 overlay data.

 Note:
       . dwL must be physical memory pointer, not virtual.
 **********************************************************/
void
dwSetOffScreenPtr_V2 (unsigned long dwL)
{
    dwOffsetOffScreen_V2 = dwL;
    pOffScreen_V2 = (unsigned char*)dwL + VGA_IO_ADDR;
}


/*----------------------------------------------
   IGS Graphics Engine Funtions

   Note: the Graphics engine routines are not necessary in overlay
	 and capture programming. All overlay and capture data
	 can be manipulated by using CPU.
  ----------------------------------------------*/

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   Basic bitblt - source copy (a color map as source)
   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*------------------------------------------------------
   copy a rectangle from src (sx, sy) to dst (dx, dy)
  ----------------------------------------------------*/
void
SrcCopy (int sx, int sy, int dx, int dy, int w, int h)
{

  unsigned long op;
  wait_for_cop_ready ();

  *Cop_control = 0;

  *Fore_Mix = (char) Mix_S;
  if (curr_bytes_per_pixel != 3)
	{
	  *Op_Dimension_1 = w - 1;
	  *Dst_Start_Ptr = (unsigned long) dy *(curr_x_ext) + dx;
	  *Src_Start_Ptr = (unsigned long) sy *(curr_x_ext) + sx;
	}
  else
	{
	  *Op_Dimension_1 = w - 1;
	  *Dst_Start_Ptr = ((unsigned long) dy * (curr_x_ext) + dx) * 3;
	  *Dst_X_Phase = ((unsigned char) dy * (curr_x_ext) + dx);
	  *Src_Start_Ptr = ((unsigned long) sy * (curr_x_ext) + sx) * 3;
	}
  *Op_Dimension_2 = h - 1;

  op = Fore_Src_PxMap;
  op |= PxBlt;
  op |= Pattern_Foreground | Right_Down;
  *Px_OpL = LOWORD (op);
  *Px_OpH = HIWORD (op);
}

/*------------------------------------------------------
   copy a rectangle from src (sx, sy) to dst (dx, dy)

   This routine play a trick to copy data in different
   dimension as the current screen resolution.

   The off-screen is used as base start pointer.

   This routine should be used together with IGS_SetPIPWindow
  ----------------------------------------------------*/
void
SrcCopy_0 (int sx, int sy, int dx, int dy, int w, int h)
{

  unsigned long op;
  wait_for_cop_ready ();

  *Cop_control = 0;

  *Fore_Mix = (char) Mix_S;
  *Op_Dimension_1 = w - 1;

  *Dst_Start_Ptr = (unsigned long) (curr_y_ext) * (curr_x_ext);
  *Dst_Start_Ptr += (unsigned long) dy *(352 * 2) + dx;

  *Src_Start_Ptr = (unsigned long) (curr_y_ext) * (curr_x_ext);
  *Src_Start_Ptr += (unsigned long) sy *(352 * 2) + sx;

  *Op_Dimension_2 = h - 1;

  op = Fore_Src_PxMap;
  op |= PxBlt;
  op |= Pattern_Foreground | Right_Down;
  *Px_OpL = LOWORD (op);
  *Px_OpH = HIWORD (op);
}



/*------------------------------------------------------
   fill a rectangle with solid color
  ----------------------------------------------------*/
void
PaintWindow (int x, int y, int w, int h, unsigned long c)
{
  unsigned long op;
  wait_for_cop_ready ();

  *Cop_control = 0;

  *Fore_Mix = (char) Mix_S;
  *Fore_Color = c;
  if (curr_bytes_per_pixel != 3)
	{
	  *Op_Dimension_1 = w - 1;
	  *Dst_Start_Ptr = (unsigned long) y *(curr_x_ext) + x;
	}
  else
	{
	  *Op_Dimension_1 = w - 1;
	  *Dst_Start_Ptr = ((unsigned long) y * (curr_x_ext) + x) * 3;
	  *Dst_X_Phase = ((unsigned char) y * (curr_x_ext) + x);
	}
  *Op_Dimension_2 = h - 1;

  op = Fore_Src_color;
  op |= PxBlt;
  op |= Pattern_Foreground | Right_Down;
  *Px_OpL = LOWORD (op);		//this can be byte, 2 bytes or 4 bytes access,

  *Px_OpH = HIWORD (op);		//but, hi byte must be the last to access.

}

/*-----------------------------------------------------
  This routine waits until Graphics Engine is not busy.
  -----------------------------------------------------*/
void
wait_for_cop_ready (void)
{
  unsigned long WaitCount;		//

  WaitCount = 0x000FFFF0;		//

  while (WaitCount != 0)		// <- with counter

	{							//

	  if ((*Cop_control & 0x80) == 0)	//

		return;					//

	  else						//

		WaitCount--;			//

	}							//

  *Cop_control = 0;				// <- force to reset

}

/*-----------------------------------------------------
  Set some Graphics Engine registers which are not
  touched by most operations.
  -----------------------------------------------------*/
void
Set_IGS_Px_Map (int w, int pixelsize)
{
  if (pixelsize != 3)
	{
	  *Src_Map_Width = w;
	  *Dst_Map_Width = w;
	}
  else							//24 bpp true color

	{
	  *Src_Map_Width = (w + 1) * 3 - 1;
	  *Dst_Map_Width = (w + 1) * 3 - 1;
	}
  *Px_Map_Format = pixelsize;
}
/**********************************************************
 Setup width and Height of video source window
       (this routine does not set hardware registers)
 in:  nWidth  = width of captured video image in pixel.
      nHeight = height of captured video image in scan line.

 return  0 if successful
	-1 if error

 note  . video 2 may not be available in some modes due to
	 off-screen memory requirment. So application should
	 make sure the offscreen has enough space to hold 2nd
	 video source data.
 **********************************************************/
int
IGS_SetSrcWindowSize_V2 (unsigned short wWidth, unsigned short wHeight)
{
  int iFlag;

  iFlag = 0;					/*to indicate successful */

  //if (wWidth % 2)
  if (wWidth & 1)
	iFlag = -1;					/*error */

  wDEFAULT_SRC_WIDTH_V2 = wWidth;
  wDEFAULT_SRC_HEIGHT_V2 = wHeight;

  return (iFlag);
}

#endif
#if 0
/**********************************************************
 Setup video 2 overlay window (destination window)
 in:  Rectangle to be overlayed to the screen:
	wLeft	(x1)
	wTop	(y1)
	wRight	(x2)
	wBottom (y2)
 **********************************************************/
void
IGS_SetOverlayWindow_V2 (unsigned short wLeft, unsigned short wTop, unsigned short wRight, unsigned short wBottom)
{
  int iDropLine, iRemainder;

  if (wRight > (unsigned short) screen.x_orig+screen.width)
	wRight = screen.x_orig+screen.width;

#if SUPPORTVUNDERSCAN
  if (wV_Underscan == 1)
	{
	  iDropLine = wTop / iMaxLines;
	  iRemainder = wTop % iMaxLines;
	  if (iRemainder > 0)
		iDropLine++;

	  wTop = wTop - iDropLine;
	}
#endif

  Out_Video_Reg (CAPTURE_BANK_CTL, 0x00);

  Out_Video_Reg (V_DST_X1_LO, LOBYTE (wLeft));
  Out_Video_Reg (V_DST_X1_HI, HIBYTE (wLeft));
  Out_Video_Reg (V_DST_X2_LO, LOBYTE (wRight));
  Out_Video_Reg (V_DST_X2_HI, HIBYTE (wRight));

  Out_Video_Reg (V_DST_Y1_LO, LOBYTE (wTop));
  Out_Video_Reg (V_DST_Y1_HI, HIBYTE (wTop));
  Out_Video_Reg (V_DST_Y2_LO, LOBYTE (wBottom));
  Out_Video_Reg (V_DST_Y2_HI, HIBYTE (wBottom));
}

#endif
